//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
//THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.
//---------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

using DesignSurfaceExtenderLibrary.Extensions;

namespace DesignSurfaceExtenderLibrary
{
    /// <summary>
    /// The designer for the <see cref="DesignSurfaceExtender"/> component.
    /// </summary>
    internal class DesignSurfaceExtenderDesigner : ComponentDesigner
    {   
        /// <summary>
        /// Backs the <see cref="ActionLists"/> property.
        /// </summary>
        private DesignerActionListCollection actionListsCollection;

        /// <summary>
        /// Backs the <see cref="Extensions"/> property.
        /// </summary>
        private ReadOnlyCollection<DesignSurfaceExtension> extensionsCollection;

        /// <summary>
        /// Path to the Plugins directory.
        /// </summary>
        /// <remarks>Change this path to the correct location for your installation.
        /// Yes, this should go into configuration.
        /// </remarks>
        private static string pluginPath = @"C:\Projects\Designer\DesignSurfaceExtenderLibrary\DesignSurfaceExtenderLibrary\Plugins";

        /// <summary>
        /// Gets the available design surface extensions.
        /// </summary>
        internal ReadOnlyCollection<DesignSurfaceExtension> Extensions
        {
            get
            {
                if( this.extensionsCollection == null )
                {
                    List<DesignSurfaceExtension> extensions = this.FindAvailableExtensions();

                    this.extensionsCollection = new ReadOnlyCollection<DesignSurfaceExtension>( extensions );
                }

                return this.extensionsCollection;
            }
        }

        private List<DesignSurfaceExtension> FindAvailableExtensions()
        {
            Assembly a = Assembly.GetExecutingAssembly();

            List<DesignSurfaceExtension> extensions = this.FindExtensionsInAssembly( a );

            string[] files = Directory.GetFiles( pluginPath );

            for( int i = 0; i < files.Length; i++ )
            {
                FileInfo fi = new FileInfo( files[i] );
                if( fi.Extension.ToLower() == ".dll" )
                {
                    a = Assembly.LoadFile( fi.FullName );
                    
                    extensions.AddRange( this.FindExtensionsInAssembly( a ) );
                }
            }

            return extensions;
        }

        /// <summary>
        /// Searches the executing assembly for types which derive 
        /// from <see cref="DesignSurfaceExtension"/>.
        /// </summary>
        /// <returns>A collection of objects which derive from 
        /// <see cref="DesignSurfaceExtension"/>.</returns>
        /// <remarks>To achieve a truly pluggable design, this method
        /// would use configuration to check other assemblies for types 
        /// which derive from <see cref="DesignSurfaceExtension"/>.
        /// </remarks>
        private List<DesignSurfaceExtension> FindExtensionsInAssembly( Assembly a )
        {
            List<DesignSurfaceExtension> extensions = new List<DesignSurfaceExtension>();

            Type[] types = a.GetTypes();

            for( int i = 0; i < types.Length; i++ )
            {
                if( types[i].IsSubclassOf( typeof( DesignSurfaceExtension ) ) )
                {
                    ConstructorInfo ci = types[i].GetConstructor(
                        BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance,
                        null,
                        new Type[] { typeof( IServiceProvider ) },
                        null );

                    if( ci != null )
                    {
                        DesignSurfaceExtension extension = ci.Invoke(
                            new object[] { this.Component.Site } ) as DesignSurfaceExtension;

                        extensions.Add( extension );
                    }
                }
            }

            return extensions;
        }

        public override DesignerActionListCollection ActionLists
        {
            get
            {
                if( null == actionListsCollection )
                {
                    actionListsCollection = new DesignerActionListCollection();

                    for( int i = 0; i < this.Extensions.Count; i++ )
                    {
                        actionListsCollection.Add( this.Extensions[i].ActionList );
                    }

                    actionListsCollection.Add( DesignSurfaceExtenderActionList.Create(
                        this.Component, this.extensionsCollection ) );
                }

                return actionListsCollection;
            }
        }
    }
}
