//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    /// <summary>
    /// A default implementation of a <see cref="Glyph"/> for the
    /// <see cref="DesignSurfaceExtenderLibrary"/>.
    /// </summary>
    public abstract class ExtensionGlyph : Glyph, IDisposable
    {
        private bool disposed;

        /// <summary>
        /// Caches a reference to the <see cref="BehaviorService"/> for convenience.  
        /// </summary>
        protected BehaviorService behaviorService;

        /// <summary>
        /// Caches a reference to the <see cref="IComponentChangeService"/> for convenience.  
        /// </summary>
        protected IComponentChangeService changeService;

        /// <summary>
        ///Caches a reference to the <see cref="ISelectionService"/> for convenience.  
        /// </summary>
        protected ISelectionService selectionService;

        /// <summary>
        /// Caches a reference to the <see cref="IReferenceService"/> for convenience.  
        /// </summary>
        protected IReferenceService referenceService;

        /// <summary>
        /// Caches a reference to an <see cref="IServiceProvider"/> for convenience.  
        /// </summary>
        protected IServiceProvider serviceProvider;

        /// <summary>
        /// Backs the <see cref="RelatedControl"/> property.
        /// </summary>
        protected Control relatedControlValue;

        /// <summary>
        /// Caches the <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.
        /// </summary>
        protected Adorner adorner;

        /// <summary>
        /// Backs the <see cref="DependentProperties"/> property.
        /// </summary>
        protected ReadOnlyCollection<string> dependentPropertiesValue;

        /// <summary>
        /// Backs the <see cref="Bounds"/> property.
        /// </summary>
        protected Rectangle boundsValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionGlyph"/> class.
        /// </summary>
        /// <param name="c">The control to be assigned as the <see cref="RelatedControl"/>. 
        /// Can be null.</param>
        /// <param name="behaviorService">A reference to the <see cref="BehaviorService"/>.</param>
        /// <param name="changeService">A reference to the <see cref="IComponentChangeService"/>.</param>
        /// <param name="selectionService">A reference to the <see cref="ISelectionService"/>.</param>
        /// <param name="referenceService">A reference to the <see cref="IReferenceService"/>.</param>
        /// <param name="serviceProvider">A reference to an <see cref="IServiceProvider"/>.</param>
        /// <param name="relatedBehavior">The <see cref="Behavior"/> which controls this <see cref="Glyph"/>.</param>
        /// <param name="adorner">The <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.</param>
        protected ExtensionGlyph(
            Control c,
            BehaviorService behaviorService,
            IComponentChangeService changeService,
            ISelectionService selectionService,
            IReferenceService referenceService,
            IServiceProvider serviceProvider,
            Behavior relatedBehavior,
            Adorner adorner )
            : base( relatedBehavior )
        {
            this.CheckParameters(
                behaviorService,
                changeService,
                selectionService,
                referenceService,
                serviceProvider,
                relatedBehavior,
                adorner );

            this.relatedControlValue = c;
            this.behaviorService = behaviorService;
            this.changeService = changeService;
            this.selectionService = selectionService;
            this.referenceService = referenceService;
            this.serviceProvider = serviceProvider;
            this.adorner = adorner;

            this.ComputeGeometry();

            this.changeService.ComponentChanged +=
                new ComponentChangedEventHandler( ComponentChangedEventHandler );
        }

        /// <summary>
        /// Validates inputs to the <see cref="ExtensionGlyph"/> constructor.
        /// </summary>
        /// <param name="behaviorService">A reference to the <see cref="BehaviorService"/>.</param>
        /// <param name="changeService">A reference to the <see cref="IComponentChangeService"/>.</param>
        /// <param name="selectionService">A reference to the <see cref="ISelectionService"/>.</param>
        /// <param name="referenceService">A reference to the <see cref="IReferenceService"/>.</param>
        /// <param name="serviceProvider">A reference to an <see cref="IServiceProvider"/>.</param>
        /// <param name="relatedBehavior">The <see cref="Behavior"/> which controls this <see cref="Glyph"/>.</param>
        /// <param name="adorner">The <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.</param>
        protected virtual void CheckParameters(
            BehaviorService behaviorService,
            IComponentChangeService changeService,
            ISelectionService selectionService,
            IReferenceService referenceService,
            IServiceProvider serviceProvider,
            Behavior relatedBehavior,
            Adorner adorner )
        {
            if( ( behaviorService == null ) ||
                ( changeService == null ) ||
                ( selectionService == null ) ||
                ( referenceService == null ) ||
                ( serviceProvider == null ) ||
                ( relatedBehavior == null ) ||
                ( adorner == null ) )
            {
                throw new ArgumentNullException();
            }
        }

        /// <summary>
        /// Calculataes the geometry of the <see cref="ExtensionGlyph"/>.
        /// </summary>
        /// <remarks>Deriving classes must override this method to 
        /// calculate the <see cref="Bounds"/>, <see cref="Control.Location"/>, and
        /// similar properties.
        /// </remarks>
        protected abstract void ComputeGeometry();

        /// <summary>
        /// Defines the collection of properties on which 
        /// the <see cref="ExtensionGlyph"/> depends. 
        /// </summary>
        /// <returns>A type-dependent collection of strings representing 
        /// the properties on which the specific <see cref="ExtensionGlyph"/> 
        /// type depends.</returns>
        protected abstract ReadOnlyCollection<string> PopulateDependentProperties();

        /// <summary>
        /// Gets a collection of strings representing the properties on which 
        /// the specific <see cref="ExtensionGlyph"/> type depends.
        /// </summary>
        protected ReadOnlyCollection<string> DependentProperties
        {
            get
            {
                if( this.dependentPropertiesValue == null )
                {
                    this.dependentPropertiesValue = this.PopulateDependentProperties();
                }

                return this.dependentPropertiesValue;
            }
        }

        /// <summary>
        /// Handles the <see cref="IComponentChangeService.ComponentChanged"/> event.
        /// </summary>
        /// <param name="sender">The <see cref="IComponent"/> which raied the event </param>
        /// <param name="e">An <see cref="EventArgs"/> which holds the event data.</param>
        protected virtual void ComponentChangedEventHandler(
            object sender,
            ComponentChangedEventArgs e )
        {
            if( object.ReferenceEquals(
                e.Component,
                this.relatedControlValue ) )
            {
                string propertyName = e.Member.Name;

                if( this.DependentProperties.Contains( propertyName ) )
                {
                    this.ComponentChangedCallback( propertyName );
                }
            }
        }

        /// <summary>
        /// Default <see cref="ExtensionGlyph"/> behavior when the given property
        /// of the related control changes.
        /// </summary>
        /// <param name="propertyName">The name of the property that changed.</param>
        protected virtual void ComponentChangedCallback( string propertyName )
        {
            this.ComputeGeometry();

            if( this.adorner.Enabled )
            {
                this.adorner.Invalidate();
            }
        }

        /// <summary>
        /// Gets the bounds of the <see cref="ExtensionGlyph"/>. 
        /// </summary>
        public override Rectangle Bounds
        {
            get
            {
                return this.boundsValue;
            }
        }

        /// <summary>
        /// The control on the design surface to which this <see cref="ExtensionGlyph"/> is bound.
        /// </summary>
        /// <remarks>Can be null for <see cref="ExtensionGlyph"/> implementations
        /// which aren't bound to particular controls.
        /// </remarks>
        public Control RelatedControl
        {
            get
            {
                return this.relatedControlValue;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        #region IDisposable Implementation

        /// <summary>
        /// Releases all resources used by the <see cref="ExtensionGlyph"/>. 
        /// </summary>
        /// <remarks>The <see cref="ExtensionGlyph"/> class implements the 
        /// the Dispose pattern in order to release event handlers.
        /// </remarks>
        public void Dispose()
        {
            Dispose( true );

            GC.SuppressFinalize( this );
        }

        /// <summary>
        /// Releases the unmanaged resources used by the <see cref="ExtensionGlyph"/> and
        /// optionally releases the managed resources. 
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; 
        /// <c>false</c> to release only unmanaged resources. </param>
        protected virtual void Dispose( bool disposing )
        {
            if( !this.disposed )
            {
                if( disposing )
                {
                    if( this.changeService != null )
                    {
                        this.changeService.ComponentChanged -=
                            new ComponentChangedEventHandler( ComponentChangedEventHandler );
                    }
                }
            }

            this.disposed = true;
        }

        #endregion IDisposable Implementation
    }
}
