Option Explicit On
Option Strict On

Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Text
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' A default implementation of an action list provided
    ''' by an <see cref="DesignSurfaceExtension"/>.
    ''' </summary>
    Public Class ExtensionActionList
        Inherits DesignerActionList
        ''' <summary>
        ''' Backs the <see cref="Extension"/> property.
        ''' </summary>
        Private extensionValue As DesignSurfaceExtension

        ''' <summary>
        ''' The name of the action list's property indicating
        ''' whether the <see cref="DesignSurfaceExtension"/> is active.
        ''' </summary>
        Private Shared propertyNameValue As String = "Enabled"

        ''' <summary>
        ''' The display text which appears on the smart tag panel. 
        ''' </summary>
        ''' <remarks>The display text is type-specific. Deriving types 
        ''' must assign this string.
        ''' </remarks>
        Protected displayTextValue As String

        ''' <summary>
        ''' This constructor is private to prevent instantiation.
        ''' </summary>
        ''' <param name="component">The related <see cref="DesignSurfaceExtender"/> component.</param>
        Private Sub New(ByVal component As IComponent)
            MyBase.New(component)

        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ExtensionActionList"/> class.
        ''' </summary>
        ''' <param name="extension">The <see cref="DesignSurfaceExtension"/> which provides this action list.</param>
        Protected Sub New(ByVal extension As DesignSurfaceExtension)
            MyBase.New(Nothing)

            Me.extensionValue = extension
        End Sub

        ''' <summary>
        ''' Returns the collection of <see cref="DesignerActionItem"/> objects 
        ''' contained in the list. 
        ''' </summary>
        ''' <returns>The type-specific list containing a <see cref="DesignerActionPropertyItem"/>
        ''' binding defined by <see cref="propertyNameValue"/> and <see cref="displayTextValue"/>
        ''' fields.</returns>
        ''' <remarks>Deriving classes must assign the <see cref="propertyNameValue"/> and
        ''' <see cref="displayTextValue"/> fields.
        ''' </remarks>
        Public Overrides Function GetSortedActionItems() As DesignerActionItemCollection
            Dim items As DesignerActionItemCollection = _
                New DesignerActionItemCollection()

            items.Add(New DesignerActionPropertyItem( _
                propertyNameValue, _
                displayTextValue))

            Return items
        End Function

        ''' <summary>
        ''' Gets the <see cref="DesignSurfaceExtension"/> which provides this action list.
        ''' </summary>
        Public ReadOnly Property Extension() As DesignSurfaceExtension
            Get
                Return Me.extensionValue
            End Get
        End Property

        ''' <summary>
        ''' Gets or sets a value indicating whether the <see cref="DesignSurfaceExtension"/> is
        ''' active.
        ''' </summary>
        ''' <remarks>Setting the <see cref="Enabled"/> property causes 
        ''' the <see cref="Adorner"/> to repaint.
        ''' </remarks>
        Public Property Enabled() As Boolean
            Get
                Return Me.Extension.Enabled
            End Get
            Set(ByVal value As Boolean)
                Me.Extension.Enabled = value

                Me.Extension.Invalidate()
            End Set
        End Property

    End Class

End Namespace
